# from pylab import *
from numpy import *
import pandas as pd
import numpy as np
import operator
import math
# import matplotlib.pyplot as plt
import random
from openpyxl import Workbook
# import array

# 数据保存在.csv文件中
df_full = pd.read_excel('NLF_A_(90).xlsx',sheetname=0,header=None)
columns = list(df_full.columns)
features = columns[:len(columns)]
# class_labels = list(df_full[columns[-1]])
df = df_full[features]
print(df)
# 维度
num_attr = len(df.columns) - 1
# 分类数
k = 3
# 最大迭代数
MAX_ITER = 1000000
# 样本数
n = len(df)  # the number of row
# 模糊参数
m = 2.00


# 初始化模糊矩阵
def initializeMembershipMatrix():
    membership_mat = list()
    for i in range(n):
        random_num_list = [random.random() for i in range(k)]
        summation = sum(random_num_list)
        temp_list = [x / summation for x in random_num_list]  # 首先归一化
        membership_mat.append(temp_list)
    return membership_mat


# 计算类中心点
def calculateClusterCenter(membership_mat):
    cluster_mem_val = zip(*membership_mat)
    cluster_centers = list()
    cluster_mem_val_list = list(cluster_mem_val)
    for j in range(k):
        x = cluster_mem_val_list[j]
        xraised = [e ** m for e in x]
        denominator = sum(xraised)
        temp_num = list()
        for i in range(n):
            data_point = list(df.iloc[i])
            prod = [xraised[i] * val for val in data_point]
            temp_num.append(prod)
        numerator = map(sum, zip(*temp_num))
        center = [z / denominator for z in numerator]  # 每一维都要计算。
        cluster_centers.append(center)
    return cluster_centers


# 更新隶属度
def updateMembershipValue(membership_mat, cluster_centers):
    #    p = float(2/(m-1))
    data = []
    for i in range(n):
        x = list(df.iloc[i])  # 取出文件中的每一行数据
        data.append(x)
        distances = [np.linalg.norm(list(map(operator.sub, x, cluster_centers[j]))) for j in range(k)]
        # print('1',distances)
        for j in range(k):
            den = sum([math.pow(float(distances[j] / distances[c]), 2) for c in range(k)])
            membership_mat[i][j] = float(1 / den)
    return membership_mat, data


# 得到聚类结果
def getClusters(membership_mat):
    cluster_labels = list()
    for i in range(n):
        max_val, idx = max((val, idx) for (idx, val) in enumerate(membership_mat[i]))
        cluster_labels.append(idx)
    return cluster_labels


def fuzzyCMeansClustering():
    # 主程序
    membership_mat = initializeMembershipMatrix()
    curr = 0
    membership_mat_old = np.zeros((n,k))
    while curr <= MAX_ITER:  # 最大迭代次数
        print('step',curr)
        cluster_centers = calculateClusterCenter(membership_mat)
        membership_mat, data = updateMembershipValue(membership_mat, cluster_centers)
        cluster_labels = getClusters(membership_mat)
        membership_mat_new = np.array(membership_mat)
        print('error',np.sum(membership_mat_new-membership_mat_old))
        if abs(np.sum(membership_mat_new-membership_mat_old)) < 0.0000000000000001:#18
            print('隶属度',membership_mat)
            break
        else:
            curr += 1
        membership_mat_old = membership_mat_new
    return cluster_labels, cluster_centers, data, membership_mat


def xie_beni(membership_mat, center, data):
    sum_cluster_distance = 0
    min_cluster_center_distance = inf
    for i in range(k):
        for j in range(n):
            sum_cluster_distance = sum_cluster_distance + membership_mat[j][i] ** 2 * sum(
                power(data[j, :] - center[i, :], 2))  # 计算类一致性
    for i in range(k - 1):
        for j in range(i + 1, k):
            cluster_center_distance = sum(power(center[i, :] - center[j, :], 2))  # 计算类间距离
            if cluster_center_distance < min_cluster_center_distance:
                min_cluster_center_distance = cluster_center_distance
    return sum_cluster_distance / (n * min_cluster_center_distance)


labels, centers, data, membership = fuzzyCMeansClustering()
print(labels)
print(centers)
center_array = array(centers)
labels = np.matrix(labels)
label = np.array(labels).T
print(label.shape)
book = Workbook()
sheet = book.create_sheet('Sheet', index=0)
[w, r] = label.shape
for i in range(w):
    for j in range(r):
        sheet.cell(row=i + 1, column=j + 1, value=label[i][j])
book.save('label_NLF_A(90).xlsx')
datas = array(data)

print("聚类有效性：",xie_beni(membership,center_array,datas))


from matplotlib import pyplot as plt
import numpy as np
import pandas as pd


"""
:param filename:
excel文件名
:return:
xArr - x数据集
yArr - y数据集
"""

m=30
for l in range(m):
    def loaddataset(filename):  # 导入EXCEL数据集
        data = pd.read_excel(filename)
        new_data = np.array(data)
        n = len(new_data)  # 矩阵的行
        x_vals = []
        y_vals = []
        for j in range(n):
            x_valss = np.array([new_data[j][i] for i in range(1)])  # 20是样本的特征（列），取的是随机样本的特征
            y_valss = np.array(new_data[j][l+1])
            print(y_valss)
            x_vals.append(x_valss)
            y_vals.append(y_valss)
        x_vals = np.array(x_vals)
        y_vals = np.array(y_vals)
        # print(x_vals)
        # print(y_vals)
        return x_vals, y_vals


    if __name__ == '__main__':
        abX, abY = loaddataset('1.xlsx')
        # plotDataSet_and_preset(abX , abY)

    from sklearn import linear_model
    from sklearn.preprocessing import PolynomialFeatures

    length = len(abX)
    # 将datasets_X转化为数组， 并变为二维，以符合线性回归拟合函数输入参数要求
    datasets_X = np.array(abX).reshape([length, 1])
    # 将datasets_Y转化为数组
    datasets_Y = np.array(abY)

    # degree=3表示建立datasets_X的二次多项式特征X_poly。
    poly_reg = PolynomialFeatures(degree=3)
    X_ploy = poly_reg.fit_transform(datasets_X)
    lin_reg_2 = linear_model.LinearRegression()
    lin_reg_2.fit(X_ploy, datasets_Y)

    # 查看回归方程系数
    print('系数：', lin_reg_2.coef_)
    # 查看回归方程截距
    print('截距：', lin_reg_2.intercept_)
    plt.scatter(datasets_X, datasets_Y, color='red')
    plt.plot(abX, lin_reg_2.predict(poly_reg.fit_transform(abX)), color='blue')
    plt.xlabel('X')
    plt.ylabel('Y')
    plt.show()
    n = 19  # 输入数据x
    arr = lin_reg_2.coef_
    YYY = lin_reg_2.intercept_ + (arr[1]) * n + (arr[2]) * (n ** 2) + (arr[3]) * (n ** 3)
    print("第",l+1,"个省份：",YYY)


# # -*- coding: utf-8 -*-
# """
# Created on Sat Aug 25 10:39:22 2018
#
# @author: aoanng
# """
# from math import log
#
# ##创建数据集
# def createDataSet():
#     """
#     创建数据集
#     """
#     dataSet = [['青年', '否', '否', '一般', '拒绝'],
#                 ['青年', '否', '否', '好', '拒绝'],
#                 ['青年', '是', '否', '好', '同意'],
#                 ['青年', '是', '是', '一般', '同意'],
#                 ['青年', '否', '否', '一般', '拒绝'],
#                 ['中年', '否', '否', '一般', '拒绝'],
#                 ['中年', '否', '否', '好', '拒绝'],
#                 ['中年', '是', '是', '好', '同意'],
#                 ['中年', '否', '是', '非常好', '同意'],
#                 ['中年', '否', '是', '非常好', '同意'],
#                 ['老年', '否', '是', '非常好', '同意'],
#                 ['老年', '否', '是', '好', '同意'],
#                 ['老年', '是', '否', '好', '同意'],
#                 ['老年', '是', '否', '非常好', '同意'],
#                 ['老年', '否', '否', '一般', '拒绝'],
#                 ]
#     featureName = ['年龄', '有工作', '有房子', '信贷情况']
#     # 返回数据集和每个维度的名称
#     return dataSet, featureName
#
# ##分割数据集
# def splitDataSet(dataSet,axis,value):
#     """
#     按照给定特征划分数据集
#     :param axis:划分数据集的特征的维度
#     :param value:特征的值
#     :return: 符合该特征的所有实例（并且自动移除掉这维特征）
#     """
#
#     # 循环遍历dataSet中的每一行数据
#     retDataSet = []
#     for featVec in dataSet:
#         if featVec[axis] == value:
#             reduceFeatVec = featVec[:axis] # 删除这一维特征
#             reduceFeatVec.extend(featVec[axis+1:])
#             retDataSet.append(reduceFeatVec)
#     return retDataSet
#
# ##计算信息熵
# # 计算的始终是类别标签的不确定度
# def calcShannonEnt(dataSet):
#     """
#     计算训练数据集中的Y随机变量的香农熵
#     :param dataSet:
#     :return:
#     """
#     numEntries = len(dataSet) # 实例的个数
#     labelCounts = {}
#     for featVec in dataSet: # 遍历每个实例，统计标签的频次
#         currentLabel = featVec[-1] # 表示最后一列
#         # 当前标签不在labelCounts map中，就让labelCounts加入该标签
#         if currentLabel not in labelCounts.keys():
#             labelCounts[currentLabel] =0
#         labelCounts[currentLabel] +=1
#
#     shannonEnt = 0.0
#     for key in labelCounts:
#         prob = float(labelCounts[key]) / numEntries
#         shannonEnt -= prob * log(prob,2) # log base 2
#     return shannonEnt
#
# ## 计算条件熵
# def calcConditionalEntropy(dataSet,i,featList,uniqueVals):
#     """
#     计算x_i给定的条件下，Y的条件熵
#     :param dataSet: 数据集
#     :param i: 维度i
#     :param featList: 数据集特征列表
#     :param unqiueVals: 数据集特征集合
#     :return: 条件熵
#     """
#     ce = 0.0
#     for value in uniqueVals:
#         subDataSet = splitDataSet(dataSet,i,value)
#         prob = len(subDataSet) / float(len(dataSet)) # 极大似然估计概率
#         ce += prob * calcShannonEnt(subDataSet) #∑pH(Y|X=xi) 条件熵的计算
#     return ce
#
# ##计算信息增益
# def calcInformationGain(dataSet,baseEntropy,i):
#     """
#     计算信息增益
#     :param dataSet: 数据集
#     :param baseEntropy: 数据集中Y的信息熵
#     :param i: 特征维度i
#     :return: 特征i对数据集的信息增益g(dataSet | X_i)
#     """
#     featList = [example[i] for example in dataSet] # 第i维特征列表
#     uniqueVals = set(featList) # 换成集合 - 集合中的每个元素不重复
#     newEntropy = calcConditionalEntropy(dataSet,i,featList,uniqueVals)#计算条件熵，
#     infoGain = baseEntropy - newEntropy # 信息增益 = 信息熵 - 条件熵
#     return infoGain
#
# ## 算法框架
# def chooseBestFeatureToSplitByID3(dataSet):
#     """
#     选择最好的数据集划分
#     :param dataSet:
#     :return:
#     """
#     numFeatures = len(dataSet[0]) -1 # 最后一列是分类
#     baseEntropy = calcShannonEnt(dataSet) #返回整个数据集的信息熵
#     bestInfoGain = 0.0
#     bestFeature = -1
#     for i in range(numFeatures): # 遍历所有维度特征
#         infoGain = calcInformationGain(dataSet,baseEntropy,i) #返回具体特征的信息增益
#         if(infoGain > bestInfoGain):
#             bestInfoGain = infoGain
#             bestFeature = i
#     return bestFeature # 返回最佳特征对应的维度
#
# def createTree(dataSet,featureName,chooseBestFeatureToSplitFunc = chooseBestFeatureToSplitByID3):
#     """
#     创建决策树
#     :param dataSet: 数据集
#     :param featureName: 数据集每一维的名称
#     :return: 决策树
#     """
#     classList = [example[-1] for example in dataSet] # 类别列表
#     if classList.count(classList[0]) == len(classList): # 统计属于列别classList[0]的个数
#         return classList[0] # 当类别完全相同则停止继续划分
#     if len(dataSet[0]) ==1: # 当只有一个特征的时候，遍历所有实例返回出现次数最多的类别
#         return majorityCnt(classList) # 返回类别标签
#     bestFeat = chooseBestFeatureToSplitFunc(dataSet)#最佳特征对应的索引
#     bestFeatLabel = featureName[bestFeat] #最佳特征
#     myTree ={bestFeatLabel:{}}  # map 结构，且key为featureLabel
#     del (featureName[bestFeat])
#     # 找到需要分类的特征子集
#     featValues = [example[bestFeat] for example in dataSet]
#     uniqueVals = set(featValues)
#     for value in uniqueVals:
#         subLabels = featureName[:] # 复制操作
#         myTree[bestFeatLabel][value] = createTree(splitDataSet(dataSet,bestFeat,value),subLabels)
#     return myTree
#
# # 测试决策树的构建
# dataSet,featureName = createDataSet()
# myTree = createTree(dataSet,featureName)
# print(myTree)

# -*- coding: utf-8 -*-
from math import log
import operator
import pickle
'''
输入：原始数据集、子数据集（最后一列为类别标签，其他为特征列）
功能：计算原始数据集、子数据集（某一特征取值下对应的数据集）的香农熵
输出：float型数值（数据集的熵值）
'''
def calcShannonEnt(dataset):
    numSamples = len(dataset)
    labelCounts = {}
    for allFeatureVector in dataset:
        currentLabel = allFeatureVector[-1]
        if currentLabel not in labelCounts.keys():
            labelCounts[currentLabel] = 0
        labelCounts[currentLabel] += 1
    entropy = 0.0
    for key in labelCounts:
        property = float(labelCounts[key])/numSamples
        entropy -= property * log(property,2)
    return entropy

'''
输入：无
功能：封装原始数据集
输出：数据集、特征标签
'''
def creatDataSet():
    dataset = [[1,1,'yes'],[1,1,'yes'],[1,0,'no'],[0,1,'no'],[0,0,'no']]
    labels = ['no surfacing','flippers']
    return dataset,labels

'''
输入：数据集、数据集中的某一特征所在列的索引、该特征某一可能取值（例如，（原始数据集、0,1 ））
功能：取出在该特征取值下的子数据集（子集不包含该特征）
输出：子数据集
'''
def getSubDataset(dataset,colIndex,value):
    subDataset = [] #用于存储子数据集
    for rowVector in dataset:
        if rowVector[colIndex] == value:
            #下边两句实现抽取除第colIndex列特征的其他特征取值
            subRowVector = rowVector[:colIndex]
            subRowVector.extend(rowVector[colIndex+1:])
            #将抽取的特征行添加到特征子数据集中
            subDataset.append(subRowVector)
    return subDataset

'''
输入：数据集
功能：选择最优的特征，以便得到最优的子数据集（可简单的理解为特征在决策树中的先后顺序）
输出：最优特征在数据集中的列索引
'''
def BestFeatToGetSubdataset(dataset):
    #下边这句实现：除去最后一列类别标签列剩余的列数即为特征个数
    numFeature = len(dataset[0]) - 1
    baseEntropy = calcShannonEnt(dataset)
    bestInfoGain = 0.0; bestFeature = -1
    for i in range(numFeature):#i表示该函数传入的数据集中每个特征
        # 下边这句实现抽取特征i在数据集中的所有取值
        feat_i_values = [example[i] for example in dataset]
        uniqueValues = set(feat_i_values)
        feat_i_entropy = 0.0
        for value in uniqueValues:
            subDataset = getSubDataset(dataset,i,value)
            #下边这句计算pi
            prob_i = len(subDataset)/float(len(dataset))
            feat_i_entropy += prob_i * calcShannonEnt(subDataset)
        infoGain_i = baseEntropy - feat_i_entropy
        if (infoGain_i > bestInfoGain):
            bestInfoGain = infoGain_i
            bestFeature = i
    return bestFeature

'''
输入：子数据集的类别标签列
功能：找出该数据集个数最多的类别
输出：子数据集中个数最多的类别标签
'''
def mostClass(ClassList):
    classCount = {}
    for class_i in ClassList:
        if class_i not in classCount.keys():
            classCount[class_i] = 0
        classCount[class_i] += 1
    sortedClassCount = sorted(classCount.iteritems(),
    key=operator.itemgetter(1),reverse = True)
    return sortedClassCount[0][0]

'''
输入：数据集，特征标签
功能：创建决策树（直观的理解就是利用上述函数创建一个树形结构）
输出：决策树（用嵌套的字典表示）
'''
def creatTree(dataset,labels):
    classList = [example[-1] for example in dataset]
    #判断传入的dataset中是否只有一种类别，是，返回该类别
    if classList.count(classList[0]) == len(classList):
        return classList[0]
    #判断是否遍历完所有的特征,是，返回个数最多的类别
    if len(dataset[0]) == 1:
        return mostClass(classList)
    #找出最好的特征划分数据集
    bestFeat = BestFeatToGetSubdataset(dataset)
    #找出最好特征对应的标签
    bestFeatLabel = labels[bestFeat]
    #搭建树结构
    myTree = {bestFeatLabel:{}}
    del (labels[bestFeat])
    #抽取最好特征的可能取值集合
    bestFeatValues = [example[bestFeat] for example in dataset]
    uniqueBestFeatValues = set(bestFeatValues)
    for value in uniqueBestFeatValues:
        #取出在该最好特征的value取值下的子数据集和子标签列表
        subDataset = getSubDataset(dataset,bestFeat,value)
        subLabels = labels[:]
        #递归创建子树
        myTree[bestFeatLabel][value] = creatTree(subDataset,subLabels)
    return myTree

'''
输入：测试特征数据
功能：调用训练决策树对测试数据打上类别标签
输出：测试特征数据所属类别
'''
def classify(inputTree,featlabels,testFeatValue):
    firstStr = inputTree.keys()[0]
    secondDict = inputTree[firstStr]
    featIndex = featlabels.index(firstStr)
    for firstStr_value in secondDict.keys():
        if testFeatValue[featIndex] == firstStr_value:
            if type(secondDict[firstStr_value]).__name__ == 'dict':
                classLabel = classify(secondDict[firstStr_value],featlabels,testFeatValue)
            else: classLabel = secondDict[firstStr_value]
    return classLabel


'''
输入：训练树，存储的文件名
功能：训练树的存储
输出：
'''
def storeTree(trainTree,filename):

    fw = open(filename,'w')
    pickle.dump(trainTree,fw)
    fw.close()
def grabTree(filename):

    fr = open(filename)
    return pickle.load(fr)


if __name__ == '__main__':
    dataset,labels = creatDataSet()
    storelabels = labels[:]#复制label
    trainTree = creatTree(dataset,labels)
    classlabel = classify(trainTree,storelabels,[0,1])
    print(classlabel)


import matplotlib.pyplot as plt
from matplotlib.pyplot import MultipleLocator
import numpy as np
import pandas as pd
def loadDataSet(filename):
    E = pd.read_excel(filename,sheetname=0,header=None)
    E = np.array(E)
    return E
X = loadDataSet('e1(mse).xlsx')
loss_list = []

for i in range(len(X)):
    for j in range(len(X[0])):
        loss_list.append(X[i][j])
x1 = range(0, 5000)
y1 = loss_list
plt.plot(x1, y1, 'o-',label="Train_Accuracy")
plt.tick_params(axis='both',which='major',labelsize=14)
x_major_locator = MultipleLocator(4)
y_major_locator = MultipleLocator(10)
ax = plt.gca()
ax.xaxis.set_major_locator(x_major_locator)
ax.yaxis.set_major_locator(y_major_locator)
plt.xlim(0,5000)
plt.ylim(0,2)
plt.title('Test loss vs. epoches')
plt.legend(loc='best')
fig = plt.figure()
ax = fig.add_subplot(111)
ax.plot(x1, y1, color='blue', linewidth=1.0, linestyle='-')
ax.xaxis.set_ticks_position('bottom')    # 将x轴刻度设在下面的坐标轴上
ax.yaxis.set_ticks_position('left')         # 将y轴刻度设在左边的坐标轴上
ax.spines['bottom'].set_position(('data', 0))   # 将两个坐标轴的位置设在数据点原点
ax.spines['left'].set_position(('data', 0))

# ax.plot(xSortAdd[:, 1], yHatAdd[srtIndAdd], color='red', linewidth=3.0, linestyle='-')

# ax.scatter(x1.flatten().A[0], y1.flatten().A[0], s=2, c='red')

# ax.axis([0, 144, 12000, 55000])
plt.xlabel('Steps')
plt.ylabel('MSE')
# year = [0,200,  400,  600,   8000]
plt.xticks(np.arange(0, 5000, 500))
# plt.yticks(np.arange(0, 0.1, 0.005))
# plt.title('House Price in West Lake, Hangzhou')
plt.show()



import numpy as np
import pandas as pd
from interval import Interval
from openpyxl import Workbook
import time
start = time.clock()
def loadDataSet(filename):
    E = pd.read_excel(filename, sheet_name=0, header=None)
    E = np.array(E)
    return E
def missing_number(a):
    c = 0
    for i in range(len(X)):
        v1 = list(a[:, 0])
        tmp_list = sorted(v1)
        c = tmp_list.count(0)   # 计算出每一列的缺失值数量
    return c

def min_max(a):
    d, b = 0, 0
    v1 = list(a[:, 0])
    tmp_list = sorted(v1)
    c = tmp_list.count(0)
    d = tmp_list[c]  # 每一列的最小值
    b = X[:, j].max()  # 每一列的最大值
    return d, b

def intervals(up,low,number):
    h = (up - low) / number
    return h

def qujian(d, t, h):
    I = np.zeros((1, t))
    u = np.zeros((1, t))
    # for s in range(t-1):
    #     I[0][s] = Interval(d + (s - 1) * h, d + s * h, lower_closed=False)
    # I[0][t] = Interval(d + (t - 1) * h, d + t * h,lower_closed=False)
    for s in range(1,t+1):
        u[0][s-1] = (d + (s - 1) * h + d + s * h) / 2
    return u

def contribution(a,t, u,  h):
    miu = np.zeros((len(X), t))
    for s in range(t):
        for i in range(len(X)):
            if a[i][0] != 0:
                if np.linalg.norm(a[i][0] - u[0][s]) <= h:
                    miu[i][s] = 1 - (np.linalg.norm(a[i][0] - u[0][s]) / h)
                else:
                    miu[i][s] = 0
            else:
                miu[i][s] = 0
    return miu

def fenjie(t, a, miu):
    m = np.zeros((len(X), t))
    for s in range(t):
        for i in range(len(X)):
            if a[i][0] != 0:
                m[i][s] = miu[i][s] * a[i][0]
    return m

def recover(t, miu,m):

    m_head = np.zeros((1, t))
    for s in range(t):
        v = 0
        n = 0
        k = 0
        o = 0
        for i in range(len(X)):
            if a[i][0] != 0:
                v = v + miu[i][s]
                o = o + m[i][s]
                n = n + a[i][0]
                k = k + 1
        if v == 0:
            m_head[0][s] = n / k
        else:
            m_head[0][s] = o / v
    return m_head

def backfill(m_head,a):
    s = 0
    for i in range(len(X)):
        if a[i][0] == 0:
            a[i][0] = m_head[0][s]
            s = s + 1
    return a


if __name__ == '__main__':
    X = loadDataSet('A(10%_33416).xlsx')  # 归一化后缺失数据集
    X_m = loadDataSet('missing_index_A(10).xlsx')
    XX1 = loadDataSet('standA.xlsx')
    number = 0
    for i in range(len(X)):
        for j in range(len(X[0])):
            if X_m[i][j] == 11:
                number += 1
    print(number)
    print(X)
    new = np.zeros((len(X), 1))
    for x in range(len(X[0])):
        # print('column',x)
        a = np.zeros((len(X), 1))
        for j in range(x, x+1):
            for i in range(len(X)):
                a[i][0] = X[i][j]
        t = missing_number(a)
        xiao, da = min_max(a)
        # print('min_max',da,xi ao)
        h = intervals(da,xiao,t)
        zhong = qujian(xiao, t, h)
        con_miu = contribution(a, t, zhong, h)
        m_combine = fenjie(t, a, con_miu)
        m_head = recover(t,con_miu,m_combine)
        a_fill = backfill(m_head,a)
        new = np.hstack((new,a_fill))
        # print('new', new,len(new),len(new[0]))
    new = np.delete(new,0,axis=1)  # 填补完的数聚集
    X_orginal = XX1
    X_estimate = new
    XXX = np.zeros((len(X_orginal),len(X_orginal[0])))
    for i in range(len(X_orginal)):
        for j in range(len(X_orginal[0])):
            if X_m[i][j] == 11: #and j != 5 and j != 4 and j != 8:
                XXX[i][j] = X_estimate[i][j]
            else:
                XXX[i][j] = X_orginal[i][j]

    QQ = X
    target = XX1
    prediction = XXX
    error = np.zeros((len(target),len(target[0])))
    for i in range(len(QQ)):
        for j in range(len(QQ[0])):
            if X_m[i][j] == 11:
                if QQ[i][j] == 0:
                    error[i][j] = target[i][j] - prediction[i][j]
                else:
                    error[i][j] = 0
    print("Errors: ", error)
    squaredError = np.zeros((len(target),len(target[0])))
    absError = np.zeros((len(target),len(target[0])))
    for i in range(len(target)):
        for j in range(len(target[0])):
            squaredError[i][j] = error[i][j] * error[i][j]
            absError[i][j] = abs(error[i][j])
    s = np.sum(squaredError)
    s_mae = np.sum(absError)
    print(s,s_mae)
    print("RMSE = ", np.sqrt(s / number))  # 均方误差MSE
    print("MSE = ", s / number)
    print("MAE = ", s_mae / number)
    book = Workbook()
    sheet = book.create_sheet('Sheet',index=0)
    [h, l] = XXX.shape
    for i in range(h):
        for j in range(l):
            sheet.cell(row=i+1, column=j+1, value=XXX[i][j])
    book.save('Fill_A(10).xlsx')  # fill完成的
end = time.clock()
t = end - start
print('Run time is :', t)










from sklearn.model_selection import KFold,StratifiedKFold
import numpy as np
import pandas as pd
from openpyxl import Workbook
import time
start = time.clock()
def loadDataSet(filename):
    E = pd.read_excel(filename,sheet_name=0,header=None)
    E = np.array(E)
    return E
def MGD_P(X,A,B,row,column,mini_batch,lamda_P):
    descentP_up = np.zeros((F, 1))
    descentP_down = np.zeros((F, 1))
    X_head = np.dot(A, B)
    for f in range(F):
        a_up = 0
        a_down = 0
        for n in range(mini_batch):
            hang = np.uint8(row[n][0])
            lie = np.uint8(column[n][0])
            a_up += X[hang][lie] * B[f][lie]
            a_down += B[f][lie] * X_head[hang][lie] + lamda_P * A[hang][f]
        descentP_up[f][0] = a_up
        descentP_down[f][0] = a_down
    for h in range(len(X)):
        for f2 in range(F):
            P[h][f2] = A[h][f2] * (descentP_up[f2][0] / descentP_down[f2][0])
    return P
def MGD_Q(X,A,B,row,column,mini_batch,lamda_Q):
    X_head1 = np.dot(A, B)
    descentQ_up = np.zeros((F, 1))
    descentQ_down = np.zeros((F, 1))
    for f3 in range(F):
        b_up = 0
        b_down = 0
        for n1 in range(mini_batch):
            hang1 = np.uint8(row[n1][0])
            lie1 = np.uint8(column[n1][0])
            b_up += A[hang1][f3] * X[hang1][lie1]
            b_down += A[hang1][f3] * X_head1[hang1][lie1] + lamda_Q * B[f3][lie1]
        descentQ_up[f3][0] = b_up
        descentQ_down[f3][0] = b_down
    for f5 in range(F):
        for t in range(len(X[0])):
            Q[f5][t] = B[f5][t] * (descentQ_up[f5][0] / descentQ_down[f5][0])
    return Q
def fill(nX,X,XX1,Xm):
    X_orginal = XX1
    X_estimate = nX
    X2 = np.zeros((len(X_orginal), len(X_orginal[0])))
    # X3 = np.zeros((len(X_orginal), len(X_orginal[0])))
    for i in range(len(X_orginal)):
        for j in range(len(X_orginal[0])):
            if Xm[i][j] == 11:
                X2[i][j] = X_estimate[i][j]
            else:
                X2[i][j] = X_orginal[i][j]
    return X2

def MSE(X_fill,X,XX,mun,Xmm):
    Y = X_fill
    Q = X
    target = XX
    prediction = Y
    error = np.zeros((len(target), len(target[0])))
    for i in range(len(Q)):
        for j in range(len(Q[0])):
            if Xmm[i][j] == 11:
                if Q[i][j] == 0:
                    error[i][j] = target[i][j] - prediction[i][j]
                else:
                    error[i][j] = 0
    squaredError = np.zeros((len(target), len(target[0])))
    # absError = np.zeros((len(target), len(target[0])))
    for i in range(len(target)):
        for j in range(len(target[0])):
            squaredError[i][j] = error[i][j] * error[i][j]
            # absError[i][j] = abs(error[i][j])
    s = np.sum(squaredError)
    MSE = s / mun
    # print("MSE = ", s / 35379)  # 均方误差MSE
    return MSE
if __name__ =='__main__':
    KF = KFold(n_splits=2)
    F = 5
    steps = 1200
    mini_batch = 2000
    list_mse = []
    list_lam =[]
    list_mseoldtrain = []
    list_lamda = []
    list_mse_train_value = []
    list_mse_test = []
    list_lam_test = []
    list_mseoldtest = []
    list_mse_test_value = []
    X = loadDataSet('A(10%_33416).xlsx')
    X_m = loadDataSet('missing_index_A(10).xlsx')
    # X1 = loadDataSet('stand_A.xlsx')
    XX1 = loadDataSet('standA.xlsx')
    # list_lamda = [0.00001, 0.0001, 0.001, 0.01, 0.1]
    # list_lamda = [0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.1]
    for a in np.arange(0,1,0.02):
        lamda_P = a
        lamda_Q = a
        print('lamda', lamda_Q)
        for train_index, test_index in KF.split(X,XX1,X_m):
            print('TRAIN:', train_index, 'TEST:', test_index)
            X_train, X_test = X[train_index], X[test_index]
            X1_train, X1_test = XX1[train_index], XX1[test_index]
            X_m_train, X_m_test = X_m[train_index], X_m[test_index]
            # train
            number_train_0 = 0
            for i in range(len(X_train)):
                for j in range(len(X_train[0])):
                    if X_m_train[i][j] == 11:
                        number_train_0 += 1
            print('11',number_train_0)
            list_mseoldtrain = []
            P1 = np.random.rand(len(X_train), F)
            Q1 = np.random.rand(F, len(X_train[0]))
            A = P1.copy()
            B = Q1.copy()
            P = np.zeros((len(X_train), F))
            Q = np.zeros((F, len(X_train[0])))
            for step in range(steps):
                # print('train_step', step)
                number = 0
                column = np.zeros((mini_batch, 1))
                row = np.zeros((mini_batch, 1))
                while number < mini_batch:
                    u = np.random.randint(0, high=len(X_train))
                    i = np.random.randint(0, high=len(X_train[0]))
                    if X_train[u][i] != 0:
                        row[number][0] = u
                        column[number][0] = i
                        number += 1
                nP = MGD_P(X_train, A, B, row, column, mini_batch, lamda_P)
                nQ = MGD_Q(X_train, A, B, row, column, mini_batch, lamda_Q)
                nX = np.dot(nP, nQ)
                X_fill = fill(nX, X_train,X1_train,X_m_train)
                mse_train = MSE(X_fill, X_train, X1_train,number_train_0,X_m_train)
                list_mseoldtrain.append(mse_train)
                print('train_mse', step,mse_train)
                A = nP.copy()
                B = nQ.copy()
            mse_min_train = min(list_mseoldtrain)
            list_mse.append(mse_min_train)  # 存储在某一lambda下每次交叉验证每一折的最小的mse值
            # test
            number_test_0 = 0
            for i in range(len(X_test)):
                for j in range(len(X_test[0])):
                    if X_m_test[i][j] == 11:
                        number_test_0 += 1
            list_mseoldtest = []
            P1_test = np.random.rand(len(X_test), F)
            Q1_test = np.random.rand(F, len(X_test[0]))
            A_test = P1_test.copy()
            B_test = Q1_test.copy()
            P = np.zeros((len(X_test), F))
            Q = np.zeros((F, len(X_test[0])))
            nE_old_test = np.linalg.norm(np.dot(P1_test, Q1_test))
            for step_test in range(steps):
                # print('step_test', step_test)
                number_test = 0
                column_test = np.zeros((mini_batch, 1))
                row_test = np.zeros((mini_batch, 1))
                while number_test < mini_batch:
                    u_test = np.random.randint(0, high=len(X_test))
                    i_test = np.random.randint(0, high=len(X_test[0]))
                    if X_test[u_test][i_test] != 0:
                        row_test[number_test][0] = u_test
                        column_test[number_test][0] = i_test
                        number_test += 1
                nP_test = MGD_P(X_test, A_test, B_test, row_test, column_test, mini_batch, lamda_P)
                nQ_test = MGD_Q(X_test, A_test, B_test, row_test, column_test, mini_batch, lamda_Q)
                nX_test = np.dot(nP_test, nQ_test)
                X_fill_test = fill(nX_test, X_test,X1_test,X_m_test)
                mse_test = MSE(X_fill_test, X_test, X1_test,number_test_0,X_m_test)
                list_mseoldtest.append(mse_test)
                print('test_mse', step_test,mse_test)
                A_test = nP_test.copy()
                B_test = nQ_test.copy()
            mse_min_test = min(list_mseoldtest)
            list_mse_test.append(mse_min_test)
    list_mse = np.matrix(list_mse)
    list_mse = np.array(list_mse).T
    print(list_mse)# 所有lambda值的训练集每一折最小的lambda值的矩阵
    list_mse_test = np.matrix(list_mse_test)
    list_mse_test = np.array(list_mse_test).T
    list_all = np.hstack((list_mse,list_mse_test))
    list_abs = np.zeros((len(list_all), 1))
    for i in range(len(list_all)):
        for j in range(len(list_all[0])):
            list_abs[i][0] = abs(list_all[i][0] - list_all[i][1])
    list_all = np.hstack((list_all, list_abs))
    book = Workbook()
    sheet = book.create_sheet('Sheet', index=0)
    [w, r] = list_all.shape
    for i in range(w):
        for j in range(r):
            sheet.cell(row=i + 1, column=j + 1, value=list_all[i][j])
    book.save('lambda-A_10.xlsx')
    # book = Workbook()
    # sheet = book.create_sheet('Sheet', index=0)
    # [w, r] = list_mse_test.shape
    # for i in range(w):
    #     for j in range(r):
    #         sheet.cell(row=i + 1, column=j + 1, value=list_mse_test[i][j])
    # book.save('A(10)_test(lambdalist).xlsx')
    end = time.clock()
    t = end - start
    print('Run time is :', t)




#         mse_tra = np.matrix(list_mse)
#         mse_tra = np.array(mse_tra).T
#         mse_tra = list(mse_tra)
#         lam = mse_tra.index(min(mse_tra))
#         mse_tra = np.array(mse_tra)
#         mse_train_value = min(mse_tra)
#         list_mse_train_value.append(mse_train_value)
#
#
#     mse_tes = np.matrix(list_mse_test)
#     mse_tes = np.array(mse_tes).T
#     mse_test_value = min(mse_tes)
#     list_mse_test_value.append(mse_test_value)
# lis_mse_tra = np.matrix(list_mse_train_value)
# mse_tr = np.array(lis_mse_tra).T
# lis_mse_tes = np.matrix(list_mse_test_value)
# mse_te = np.array(lis_mse_tes).T











import pandas as pd
import numpy as np
import math
import random
from numpy import *
from openpyxl import Workbook
# 导入数据集
def LoadDataSet(filename):
    E = pd.read_excel(filename,sheet_name=0,header=None)
    E = np.array(E)
    return E

# 初始化特征权重矩阵
def initializefeature_weightMatrix():
    feature_weight_mat = np.full((k, number_attribute), (1/number_attribute))
    return feature_weight_mat

# 初始化隶属度矩阵
def initializeMembershipMatrix():
    membership_mat = list()
    for i1 in range(n):
        random_num_list = [random.random() for q in range(k)]
        summation = sum(random_num_list)
        temp_list = [x / summation for x in random_num_list]  # 归一化，保证和等于1
        membership_mat.append(temp_list)
    return membership_mat  # 返回隶属度矩阵

# 计算聚类中心
def calculateClusterCenter(X, Z,Membership_mat):
    cluster_centers = np.zeros((k,number_attribute))
    cluster_centers_z = np.zeros((k, n))
    centers_z = []
    centers = []
    mem_mat = np.zeros((n,k))
    for i2 in range(n):
        for j2 in range(k):
            mem_mat[i2][j2] = np.power(Membership_mat[i2][j2], m)  # 计算隶属度^m 的值
    # 计算样本集的中心
    for kkk in range(k):
        temp_num = list()
        for i10 in range(n):
            data_point = list(X[i10,:])
            prod = [mem_mat[i10][kkk] * val for val in data_point]
            temp_num.append(prod)
        numerator = map(sum,zip(*temp_num))
        center = [x / sum(mem_mat[:,kkk]) for x in numerator]
        centers.append(center)
    cluster_centers = np.array(centers)
    # 计算判别矩阵的中心
    for kk in range(k):
        temp_num_z = list()
        for i6 in range(n):
            data_point_z = list(Z[i6,:])  # 读取稀疏自表示的第i行
            prod_z = [mem_mat[i6][kk] * val_z for val_z in data_point_z]
            temp_num_z.append(prod_z)
        numerator_z = map(sum, zip(*temp_num_z))
        center_z = [z / sum(mem_mat[:,kk]) for z in numerator_z]
        centers_z.append(center_z)
    cluster_centers_z = np.array(centers_z)
    return cluster_centers, cluster_centers_z

# 更新隶属度
def updateMembershipValue(Membership_mat, cluster_centers, cluster_centers_z):
    for i4 in range(n):
        for c in range(k):
            up = np.power((np.linalg.norm(X[i4,:]-cluster_centers[c,:])),2) + alpha * np.power(np.linalg.norm(Z[i4,:] - cluster_centers_z[c,:]),2)
            down = 0
            down_attr = 0
            for l in range(k):
                down_attr += 1 / (np.power((np.linalg.norm(X[i4,:]-cluster_centers[l,:])),2) + alpha * np.power(np.linalg.norm(Z[i4,:] - cluster_centers_z[l,:]),2))
            # print(i4,c,up,down,up/down)
            Membership_mat[i4][c] = 1 / (up * down_attr)
    return Membership_mat

# 得到聚类结果
def getclusters(Membership_mat):
    cluster_labels = list()
    # Membership_mat = list(Membership_mat)
    for i in range(n):
        max_val, idx = max((val, idx) for (idx, val) in enumerate(Membership_mat[i]))
        cluster_labels.append(idx)
    return cluster_labels

# 聚类指标
def xie_beni(membership_mat, center, center_z, data, data_z):
    sum_cluster_distance = 0
    min_cluster_center_distance = inf
    membership_mat = np.array(membership_mat)
    data = np.array(data)
    data_z = np.array(data_z)
    for i in range(k):
        for j in range(n):
            sum_cluster_distance = sum_cluster_distance + membership_mat[j][i] ** 2 * ((math.pow(np.linalg.norm(data[j, :] - center[i, :]), 2)) + alpha * (math.pow(np.linalg.norm(data_z[j, :] - center_z[i, :]), 2)))  # 计算类一致性
    for i in range(k - 1):
        for j in range(i + 1, k):
            cluster_center_distance = math.pow(np.linalg.norm(center[i, :] - center[j, :]), 2) + alpha * math.pow(np.linalg.norm(center_z[i, :] - center_z[j, :]), 2)  # 计算类间距离
            if cluster_center_distance < min_cluster_center_distance:
                min_cluster_center_distance = cluster_center_distance
    return sum_cluster_distance / (n * min_cluster_center_distance)


if __name__ == '__main__':
    X = LoadDataSet('NLF_A_(90).xlsx')
    Z = LoadDataSet('distance_NLF_A(90).xlsx')
    print(Z.shape)
    # Z2 = LoadDataSet('distanceF_1.xlsx')
    # Z = np.hstack((Z1,Z2))
    # print(Z2.shape)
    # X = np.random.rand(6,3)
    # print(X)
    # Z = [[0.1, 0.02, 0.6, 0.5, 0.9, 0.0001],
    #      [0.02, 0.8, 0.4, 0.04, 0.003, 0.05],
    #      [0.6, 0.4, 0.008, 0.1, 0.06, 0.0],
    #      [0.5, 0.004, 0.008, 0.02, 0.4, 0.000001],
    #      [0.9, 0.003, 0.1, 0.06, 0.4, 0.2],
    #      [0.0001, 0.05, 0.00, 0.000001, 0.2, 0.6]]
    # Z = np.array(Z)
    n = len(X)  # 样本数
    number_attribute = len(X[0])  # 特征数
    m = 5  # 模糊系数
    k = 3  # 聚类数
    alpha = 0.8  # 判别矩阵的影响系数
    step = 0
    max_step = 10000
    membership_mat_old = np.zeros((n, k))
    Membership_mat = initializeMembershipMatrix()
    print(Membership_mat)
    # Feature_weight_mat = initializefeature_weightMatrix()
    while step <= max_step:
        print('step', step)
        cluster_centers, cluster_centers_z = calculateClusterCenter(X, Z, Membership_mat)
        Membership_mat = updateMembershipValue(Membership_mat, cluster_centers, cluster_centers_z)
        # Feature_weight_mat = updatefeatureweight(Membership_mat,X,cluster_centers)
        cluster_labels = getclusters(Membership_mat)
        membership_mat_new = np.array(Membership_mat)
        print('error', abs(np.sum(membership_mat_new - membership_mat_old)))
        if abs(np.sum(membership_mat_new - membership_mat_old)) < 0.0000000000000001:
            print('隶属度', Membership_mat)
            break
        else:
            step += 1
        membership_mat_old = membership_mat_new
    labels = getclusters(Membership_mat)
    labels = np.matrix(labels)
    label = np.array(labels).T
    print(label.shape)
    book = Workbook()
    sheet = book.create_sheet('Sheet', index=0)
    [w, r] = label.shape
    for i in range(w):
        for j in range(r):
            sheet.cell(row=i + 1, column=j + 1, value=label[i][j])
    book.save('labelD_NLF_A(90).xlsx')
    print(labels)
    XB = xie_beni(Membership_mat,cluster_centers,cluster_centers_z,X,Z)
    print('聚类有效性',XB)



